"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.responseContext = void 0;
const validateDto_1 = require("../decorators/validation/helpers/validateDto");
class ResponseContext {
    constructor() { }
    static get Instance() {
        if (!ResponseContext._Instance)
            ResponseContext._Instance = new ResponseContext();
        return ResponseContext._Instance;
    }
    createResponseHandler(propName, controllerInstance) {
        const getArgs = this.addRequiredHandlerArguments.bind(this);
        const handleResponse = this.handleResponse.bind(this);
        return function (req, res, next) {
            try {
                let methodArguments = [];
                const argEntity = Reflect.getMetadata("method:param", controllerInstance, propName);
                if (argEntity) {
                    const params = argEntity.argModel;
                    const paramDataTypes = Reflect.getMetadata("design:paramtypes", controllerInstance, propName);
                    methodArguments = [...methodArguments, ...getArgs(params, paramDataTypes, req, res, next)];
                }
                let methodResult;
                methodResult = controllerInstance[propName].apply(controllerInstance, methodArguments);
                if (methodResult instanceof Promise) {
                    methodResult
                        .then((promiseResult) => {
                        handleResponse(res, promiseResult);
                    })
                        .catch((error) => {
                        return next(error);
                    });
                }
                else {
                    handleResponse(res, methodResult);
                }
            }
            catch (error) {
                return next(error);
            }
        };
    }
    createErrorResponseHandler(propName, controllerInstance) {
        const getArgs = this.addRequiredHandlerArguments.bind(this);
        const handleResponse = this.handleResponse.bind(this);
        return function (error, req, res, next) {
            try {
                let methodArguments = [];
                const argEntity = Reflect.getMetadata("method:param", controllerInstance, propName);
                if (argEntity) {
                    const params = argEntity.argModel;
                    const paramDataTypes = Reflect.getMetadata("design:paramtypes", controllerInstance, propName);
                    methodArguments = [...methodArguments, ...getArgs(params, paramDataTypes, req, res, next, error)];
                }
                let methodResult;
                methodResult = controllerInstance[propName].apply(controllerInstance, methodArguments);
                if (methodResult instanceof Promise) {
                    methodResult
                        .then((promiseResult) => {
                        handleResponse(res, promiseResult);
                    })
                        .catch((error) => {
                        return next(error);
                    });
                }
                else {
                    handleResponse(res, methodResult);
                }
            }
            catch (error) {
                return next(error);
            }
        };
    }
    handleResponse(res, methodResult) {
        if (!res.headersSent) {
            switch (typeof methodResult) {
                case "string":
                    res.send(methodResult);
                    return;
                case "object":
                    res.json(methodResult);
                    return;
                case "bigint":
                case "boolean":
                case "number":
                    res.send(methodResult.toString());
                    return;
                case "undefined":
                    break;
                default:
                    res.end();
                    return;
            }
        }
    }
    addRequiredHandlerArguments(params, paramDataTypes, req, res, next, error) {
        const args = [];
        for (let index = 0; index < params.length; index++) {
            const param = params[index];
            let paramValidationSuccess = true;
            switch (param.type) {
                case "context":
                    if (error)
                        args.push({ error, req, res, next });
                    else
                        args.push({ req, res, next });
                    break;
                case "error":
                    args.push(error);
                    break;
                case "req":
                    args.push(req);
                    break;
                case "res":
                    args.push(res);
                    break;
                case "next":
                    args.push(next);
                    break;
                case "body":
                    paramValidationSuccess = validateDto_1.validateDto(req.body, paramDataTypes[index]);
                    args.push(param.key ? req.body[param.key] : req.body);
                    break;
                case "param":
                    args.push(param.key ? req.params[param.key] : req.params);
                    break;
                case "query":
                    args.push(param.key ? req.query[param.key] : req.query);
                    break;
            }
            if (!paramValidationSuccess)
                throw new Error("Invalid DTO");
        }
        return args;
    }
}
ResponseContext._Instance = null;
exports.responseContext = ResponseContext.Instance;
//# sourceMappingURL=ResponseContext.js.map